@@ -7,16 +7,17 @@ source tests/buildkite/conftest.sh
 echo "--- Run Google Tests with CUDA, using a GPU"
 buildkite-agent artifact download "build/testxgboost" . --step build-cuda
 chmod +x build/testxgboost
-tests/ci_build/ci_build.sh gpu nvidia-docker \
+tests/ci_build/ci_build.sh gpu --use-gpus \
   --build-arg CUDA_VERSION_ARG=$CUDA_VERSION \
   --build-arg RAPIDS_VERSION_ARG=$RAPIDS_VERSION \
+  --build-arg NCCL_VERSION_ARG=$NCCL_VERSION \
   build/testxgboost
 
 echo "--- Run Google Tests with CUDA, using a GPU, RMM enabled"
 rm -rfv build/
 buildkite-agent artifact download "build/testxgboost" . --step build-cuda-with-rmm
 chmod +x build/testxgboost
-tests/ci_build/ci_build.sh gpu nvidia-docker \
+tests/ci_build/ci_build.sh gpu --use-gpus \
   --build-arg CUDA_VERSION_ARG=$CUDA_VERSION \
   --build-arg RAPIDS_VERSION_ARG=$RAPIDS_VERSION \
   --build-arg NCCL_VERSION_ARG=$NCCL_VERSION \
